// string_tool.h (C) 2012 adolfo@di-mare.com

/** \file   string_tool.h
    \brief  Rutinas de hileras que complementan \c \<string> y \c \<cstring>.
    \author Adolfo Di Mare <adolfo@di-mare.com>
    \date   2012
*/

#ifndef    string_tool_h
#define    string_tool_h

/// Rutinas \c string_tool.h de hileras que complementan \c \<string> y \c \<cstring>.
struct string_tool{}; ///< - Truco para que salga como clase en Doxygen.

#include <list>    // std::list
#include <string>  // std::string
#include <cstring> // std::strlen()

void tolower(  std::string& line );
char removeAccented( char ch );
void trimQuote( std::string& word );
const char* removeAccented( char* str );
const std::string& removeAccented( std::string& str );
bool isalpha(  const std::string& line );
bool is_blank( const std::string& word );
bool issuffix( const char* str, const char* suffix );

unsigned digitCount( const std::string& str , unsigned i );
unsigned alphaCount( const std::string& str , unsigned i );
unsigned number_suffix( const std::string& str );
unsigned str2uint( const std::string& str );
std::string tostring( unsigned n, unsigned width );

/// Retorna \c true si \c es uno de { '-' '_' '~' }.
inline bool isdash( char c ) { return c=='-' || c=='_' || c=='~'; }

/** Retorna \c true si \c ch es un caracter son letras pueden aparecer en una direccin email.
    - Es equivalente a <code>( isalpha(ch) || ch=='-' || ch=='.'  || ch=='_' )</code>

    \dontinclude test_tool.cpp
    \skipline    test::is_email()
    \until       }}
*/
inline bool is_email( char ch ) {
    return ( isalpha(ch) || ch=='-' || ch=='.'  || ch=='_' );
}

void tokens( const std::string& renglon, std::list<std::string>& L, const char* delimiters );

#endif  // string_tool_h

// EOF: string_tool.h
