/* test_tool.cpp  (C) 2012  adolfo@di-mare.com  */

/** \file  test_tool.cpp
    \brief Programa de prueba para las rutinas de \c string_tool.h.

    \author Adolfo Di Mare <adolfo@di-mare.com>
    \date   2012
*/

#include "uUnit.h"   // assertTrue()+assertFalse()
#include <iostream>  // std::cout

#include "string_tool.h"
#include "psz_string.h"    // recuerda delete[] al usar (char*)

/// \c test->trimQuote().
void test_trimQuote() {
    using namespace std;
{{  // test::trimQuote()
    string  A = "\'A\'"; trimQuote(A);
    assertTrue( "A"==A );

    string  B = "\"B\'"; trimQuote(A);
    assertTrue( "B"==B );

    string  C = "\"C\""; trimQuote(A);
    assertTrue( "C"==C );

    string       ABC123XYZ =  "ABC123XYZ";
    trimQuote(   ABC123XYZ );
    assertTrue( "ABC123XYZ" == ABC123XYZ );
}}
}

/// \c test->tolower().
void test_tolower() {
    using namespace std;
{{  // test::tolower()
    string  A = "A"; tolower(A);
    assertTrue( "a"==A );

    string       ABC123XYZ =  "ABC123XYZ";
    tolower(     ABC123XYZ );
    assertTrue( "abc123xyz" == ABC123XYZ );
}}
    string UP = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"; tolower(UP);
    assertTrue( "abcdefghijklmnopqrstuvwxyz" == UP );
}

/// test->removeAccented() && test->removeAccented::string()
void test_removeAccented() {
    using namespace std;
{{  // test::removeAccented(string)
    string Atilde  = "";
    string Asimple = "AAAAAAECEEEEIIIIDNOOOOOx0UUUUYPsaaaaaaeceeeeiiiiOnooooo/0uuuuypy";

    string Astr = Atilde;
    assertTrue( Asimple == removeAccented(Astr) );
}}
{{  // test::removeAccented(char*)
    char Atilde[]  = "";
    char Asimple[] = "AAAAAAECEEEEIIIIDNOOOOOx0UUUUYPsaaaaaaeceeeeiiiiOnooooo/0uuuuypy";

    char Astr[ sizeof(Atilde) ];
    strcpy( Astr, Atilde );

    assertTrue( 0==strcmp( Asimple, removeAccented(Astr) ) );
}}
{{  // test::removeAccented(char)
    char Atilde[]  = "";
    char Asimple[] = "AAAAAAECEEEEIIIIDNOOOOOx0UUUUYPsaaaaaaeceeeeiiiiOnooooo/0uuuuypy";

    assertTrue( 'A' == removeAccented('') );    assertTrue( 'N' == removeAccented('') );
    assertTrue( 'Y' == removeAccented('') );    assertTrue( 'D' == removeAccented('') );

    assertTrue( 's' == removeAccented('') );    assertTrue( 'P' == removeAccented('') );
    assertTrue( 'x' == removeAccented('') );    assertTrue( '/' == removeAccented('') );
    assertTrue( '0' == removeAccented('') );    assertTrue( 'O' == removeAccented('') );

    assertTrue( Atilde != Asimple );
}}
{
    // Copia del cdigo que est arriba
    const char *Atilde  = "";
    const char *Asimple = "AAAAAAECEEEEIIIIDNOOOOOx0UUUUYPsaaaaaaeceeeeiiiiOnooooo/0uuuuypy";

    char Astr[ 255-192+1 ]; *Astr = 0;
    strcpy( Astr, Atilde );

    assertTrue( 0==strcmp( Asimple, removeAccented(Astr) ) );
    assertTrue( strlen(Asimple)==strlen(Astr) );

    #ifdef __GNUC__  // Esto es raro ...
        assertTrue(  (unsigned char)('')==192 );
        assertFalse(          (char)('')==192 );
        assertFalse( (unsigned)     ('')==192 );
    //  http://gcc.gnu.org/onlinedocs/cpp/Common-Predefined-Macros.html
    #endif

//  std::cout << '' << "\n" << Atilde << "\n" << Astr << "\n";
}
{
    // verifica que la implementacin de removeAccented(char) funciona
    const char *Atilde  = "";
    const char *Asimple = "AAAAAAECEEEEIIIIDNOOOOOx0UUUUYPsaaaaaaeceeeeiiiiOnooooo/0uuuuypy";

    char Astr[ 255-192+1 ];
    strcpy( Astr, Atilde );

    char *p = Astr;
    while ( (*p)!=0 ) {
        *p = removeAccented(*p) ;
        ++p;
        // http://stackoverflow.com/questions/14094621/
    }
    assertTrue( 0==strcmp( Asimple, removeAccented(Astr) ) );
    assertTrue( strlen(Asimple)==strlen(Astr) );
    assertTrue( 'n' == removeAccented('') );
}
}

/// \c test->isalpha().
void test_isalpha() {
{{  // test::isalpha()
    assertTrue(  isalpha("A") );
    assertTrue(  isalpha("abcABCxyz") );
    assertFalse( isalpha("123abc456") );
}}
    assertTrue( isalpha("abcdefghijklmnopqrstuvwxyz") );
    assertTrue( isalpha("ABCDEFGHIJKLMNOPQRSTUVWXYZ") );
}

/// \c test->is_blank().
void test_is_blank() {
{{  // test::is_blank()
    assertTrue(  is_blank(" \t\r\n\v") );
    assertFalse( is_blank(" ? ") );
}}
}

/// \c test->digitCount().
void test_digitCount() {
    using namespace std;
{{  // test::digitCount()
    string sAE = "ABCDE";
    string s15 = "12345";
    assertTrue( digitCount("12345",0)==5 && s15[0]=='1' );
    assertTrue( digitCount("12345",4)==1 && s15[4]=='5' );
    assertTrue( digitCount("12345",5)==string::npos );  // fuera de rango
    assertTrue( digitCount("ABCDE",0)==0 );
}}
    assertTrue( "12345"[0]=='1' );
    assertTrue( "12345"[4]=='5' );
    assertTrue( digitCount("12345",0)==5 && "12345"[0]=='1' );
    assertTrue( digitCount("12345",4)==1 && "12345"[4]=='5' );
    assertTrue( digitCount("12345",5)==string::npos && "12345"[5]==0 );
    assertTrue( digitCount("12345",6)==string::npos );
    assertTrue( digitCount(string(),0)==string::npos );

    string s15 = "12345"; s15 += '.';
    assertTrue( digitCount(s15,5)==0 && s15[5]=='.' );

    assertTrue( digitCount("--3",0)==0 );
    assertTrue( digitCount( "-3",0)==0 );
    assertTrue( digitCount(  "3",0)==1 );
    assertTrue( digitCount("++3",0)==0 );
    assertTrue( digitCount( "+3",0)==0 );
    assertTrue( digitCount(  "3",0)==1 );
}

/// \c test->alphaCount().
void test_alphaCount() {
    using namespace std;
{{  // test::alphaCount()
    assertTrue( alphaCount("ABCDE",0)==5  && "ABCDE"[0]=='A' );
    assertTrue( alphaCount("ABCDE",4)==1  && "ABCDE"[4]=='E' );
    assertTrue( alphaCount("ABCDE",5)==string::npos );   // fuera de rango
    assertTrue( alphaCount("12345",0)==0 );
}}
//
    string s15 = "12345";
    assertTrue( alphaCount("ABCDE",0)==5 && "ABCDE"[0]=='A' );
    assertTrue( alphaCount("ABCDE",4)==1 && "ABCDE"[4]=='E' );

    string sAE = "ABCDE";    sAE += '.';
    assertTrue( alphaCount(sAE,5)==0 && sAE[5]=='.' );
    assertTrue( alphaCount(sAE,6)==string::npos );
    assertTrue( alphaCount(string(),0)==string::npos );
}

/// \c test->roundUp().
void test_roundUp() {
    #undef roundUp
    // no_roundUp(n,b) ( (n)%(b)==0 ? n : (b)*( (n)/(b) )+(b) )
    // no_roundUp(n,b) ( (n)+(b) - (n)%(b) )

    /// Rounding up to the nearest multiple of a number.
    /// - Not tested for negative numbers.
    /// \see http://stackoverflow.com/questions/3407012/
    #define roundUp(n,b) ( (b)==0 ? (n) : ( ((n)+(b)-1) - (((n)-1)%(b)) ) )

    // yes_roundUp(n,b) ( (b)==0 ? (n) : ( (n)%(b)==0 ? n : (n)+(b)-(n)%(b) ) )
    // yes_roundUp(n,b) ( (b)==0 ? (n) : ( ((n + b - 1) / b) * b ) )

if (true) // todava no funciona sin (?:)
{{  // test::roundUp()
    unsigned m;
                { m = roundUp(17,8); } ++m;
    assertTrue( 24 == roundUp(17,8) );
                { m = roundUp(24,8); }
    assertTrue( 24 == roundUp(24,8) );

    assertTrue( 24 == roundUp(24,4) );
    assertTrue( 24 == roundUp(23,4) );
                { m = roundUp(23,4); }
    assertTrue( 24 == roundUp(21,4) );

    assertTrue( 20 == roundUp(20,4) );
    assertTrue( 20 == roundUp(19,4) );
    assertTrue( 20 == roundUp(18,4) );
    assertTrue( 20 == roundUp(17,4) );

    assertTrue( 17 == roundUp(17,0) );
    assertTrue( 20 == roundUp(20,0) );
}}
    #undef roundUp
}

/// \c test->issuffix().
void test_issuffix() {
    using namespace std;
{{  // test::issuffix()
    assertTrue(  issuffix("la mano alzada","ALZADA" ) );
    assertFalse( issuffix("ALZADA","la mano alzada" ) );
    assertTrue(  issuffix("alzada","ALZADA" ) );
    assertTrue(  issuffix(0,"") );
    assertTrue(  issuffix("",0) ); // NULL
}}
    assertTrue(  issuffix(0,0) );
    assertTrue(  issuffix("","") );
    assertTrue(  issuffix("alzada","" ) );

    assertTrue(  issuffix("121","1" ) );
    assertFalse( issuffix("1","121" ) );

    assertTrue(  issuffix("12321","21" ) );
    assertFalse( issuffix("21","12321" ) );
    {
        const char *filename ,*name_cfg;
        filename = "yy-VACIO-bt0pp.cfg";
        name_cfg =   "Xvacio-bt0pp.cfg";
        assertFalse( issuffix( filename , name_cfg ) );
        filename = "yy-VACIO-bt0pp.cfg";
        name_cfg =    "vacio-bt0pp.cfg";
        assertTrue( issuffix( filename , name_cfg ) );
    }
    {
        const char *filename ,*name_cfg;
        filename = "00-bt0pp.cfg";
        name_cfg =    "bt0pp.cfg";
        assertTrue(  issuffix( filename , name_cfg ) );
        filename = "00-bt0pp.cfg xxx";
        assertFalse( issuffix( filename , name_cfg ) );
    }
    assertFalse( issuffix("Xvacio-bt0pp.cfg","yy-VACIO-bt0pp.cfg" ) );
    assertFalse( issuffix("Xvacio-bt0pp.cfg","zz-VACIO-bt0pp.cfg" ) );

    assertTrue( issuffix("yy-VACIO-bt0pp.cfg","vacio-bt0pp.cfg" ) );
    assertTrue( issuffix("zz-VACIO-bt0pp.cfg","vacio-bt0pp.cfg" ) );
}

/// \c test->str2uint().
void test_str2uint() {
{{  // test::str2uint()
    assertTrue(  123 == str2uint("+123" ) );
    assertTrue(  123 == str2uint("-123" ) );
    assertTrue(    0 == str2uint("") );
    assertTrue(    0 == str2uint("+-") );
    assertTrue( 2012 == str2uint("2012.3.etc...") );
}}
}

/// Test->psz_string().
void test_psz_string() {
{{  // test::psz_string()
    const char * str = "local/global";
    psz_string psz = str;   assertTrue( 0!=psz );
    psz.clear();            assertTrue( 0==psz );

    psz = str; // si copia contenido
    assertTrue( psz!=str && "memoria differente");
    assertTrue( 0==strcmp(psz,str) && "contienen lo mismo");

    try {
        psz_string NESTED; assertTrue( 0==NESTED );
        NESTED.alloc( 1+strlen(psz) ); // +1 ==> EOS del final
        strcpy( NESTED, psz );
        assertTrue( 0==strcmp(NESTED,str) && "contienen lo mismo");
        throw int(32); // throw tira para que catch() apae
    }
    catch ( int val ) {
        assertTrue( "la memoria de [NESTED] ha sido liberada" );
        assertTrue( val==32 );
    }

    psz.alloc(0); // libera memoria
    assertTrue( 0==psz );
}}
{
    psz_string psz( strlen("banana") );
    strcpy( psz, "banana");
    assertTrue( 0==strcmp("banana",psz) );
}
{   // auto-copia
    psz_string a,b,c;
    a = "aaaaa";
    b = "bbb";
    c = "c";

    a = a;            assertTrue( 0==strcmp(a , "aaaaa") && "auto-copia" );
    b = (char*)(b);   assertTrue( 0==strcmp(b , "bbb")   && "auto-copia" );
    c = 0;            assertTrue( c == 0 );
}
}

void test_number_suffix() {
{{  // test::number_suffix()
    assertTrue(   1==number_suffix("1") );
    assertTrue(   1==number_suffix("grupo001") );
    assertTrue( 123==number_suffix("g123") );
    assertTrue( 123==number_suffix("-123") );
    assertTrue( std::string::npos==number_suffix("") );
    assertTrue( std::string::npos==number_suffix("132xx") );
}}
    assertTrue(    0==number_suffix("000") );
    assertTrue(    0==number_suffix("g0") );
    assertTrue(    0==number_suffix("0") );
    assertTrue(    0==number_suffix("-0") );
    assertTrue( 1125==number_suffix("1125") );
}

/// \c test->tostring().
void test_tostring() {
{{  // test::tostring()
    assertTrue( "001" == tostring(1, 3) );
    assertTrue( "789" == tostring(123456789, 3) );
    assertTrue(    "" == tostring(123456789, 0) ); // 0==>""
}}
    assertTrue(   "1" == tostring(1, 1) );
    assertTrue(   "0" == tostring(0, 1) );
    assertTrue( "000" == tostring(0, 3) );
    assertTrue(  "89" == tostring(123456789, 2) );
    assertTrue(    "" == tostring(123456789, 0) ); // 0==>""
}

/// \c test->is_email().
void test_is_email() {
{{  // test::is_email()
    assertFalse( is_email('@') );
    const char* yo = "adolfo@di-mare.com";
    std::string YO = yo;

    size_t prev = YO.find('@');
    for ( size_t i=0; i<prev; ++i ) {
        assertTrue( is_email( YO[i] ) );
    }

    size_t post = prev+1;
    for ( size_t i=post; i<YO.size(); ++i  ) {
        assertTrue( is_email( YO[i] ) );
    }
}}
}

/// \c test->tokens().
void test_tokens() {
{{  // test::tokens()
    std::string ln;

    ln = "  gsec =##-t|##-tpg (mejor peor);   ;?-###   ;etc." ;
    //         1 2   34     5 6    7 8   910 11   12  13  14
#if 0
    [gsec] [=] [##-t] [|] [##-tpg] [(] [mejor] [peor] [)] [;] [;] [?-###] [;] [etc.]
        1   2      3   4        5   6       7      8   9   10  11      12  13    14
#endif

    std::list< std::string > LLL;
    std::list< std::string >::const_iterator it;
    tokens( ln, LLL , " =();|" );

    assertTrue( 14 == LLL.size() );  assertTrue( 12 != LLL.size() );
    it = LLL.begin();   assertTrue( *it == "gsec" );
    ++it;               assertTrue( *it == "=" );
    ++it;               assertTrue( *it == "##-t" );

    it = LLL.begin(); for (int i=1; i<6;  ++i,it++) { } assertTrue( *it == "(" );
    it = LLL.begin(); for (int i=1; i<10; ++i,it++) { } assertTrue( *it == ";" );
    it = LLL.begin(); for (int i=1; i<12; ++i,it++) { } assertTrue( *it == "?-###" );
    {   // puros blancos
        std::list< std::string > LLL;
        std::list< std::string >::const_iterator it;
        std::string ln;
        ln = "      " ; tokens( ln, LLL , " ;" ); // puros blancos...
        assertTrue( LLL.empty() );
        ln = " ; etc "; tokens( ln, LLL , " ;" ); // puros blancos...
        assertTrue( LLL.size()==2 && LLL.front()==";" );
    }
}}
{
    std::string ln;
    ln = "gsec=##-t|##-tpg(mejor peor);   ;?-###;etc." ;
    //       12   34     56    7 8   910 11   1213 14
#if 0
    [gsec] [=] [##-t] [|] [##-tpg] [(] [mejor] [peor] [)] [;] [;] [?-###] [;] [etc.]
        1   2      3   4        5   6       7      8   9   10  11      12  13    14
#endif
    std::list< std::string > LLL;
    std::list< std::string >::const_iterator it;
    tokens( ln, LLL , " =();|" );
    #if 0
        assertFalse( " *** Por que NO detecta '|' ni ')' ??? *** " );
        for ( it=LLL.begin(); it!=LLL.end(); ++it) {
            std::cout<< "[" << *it << "] ";
        }
        std::cout << std::endl;
    #endif
    assertTrue( 14 == LLL.size() );  assertTrue( 12 != LLL.size() );
    it = LLL.begin(); for (int i=1; i<6;  ++i,it++) { } assertTrue( *it == "(" );
    it = LLL.begin(); for (int i=1; i<10; ++i,it++) { } assertTrue( *it == ";" );
    it = LLL.begin(); for (int i=1; i<12; ++i,it++) { } assertTrue( *it == "?-###" );
}

{   // Ms pruebas para tokens()
    std::string ln;
    //       0123     45  67     89x    11x    13x 15x        17x  19
    ln = "2012.1.ci1101.g04.adolfo..dimare.examen.ex2.1-enunciado.htm" ;
    std::list< std::string > LLL;
    std::list< std::string >::const_iterator it;
    tokens( ln, LLL , " ." );
    #if 0
        unsigned i=0;
        for ( it=LLL.begin(); it!=LLL.end(); ++it,++i) {
            std::cout<< i << "[" << *it << "] ";
        }
        std::cout << std::endl;
    #endif
    assertTrue( "htm" == LLL.back() );
    it = LLL.begin(); for (int i=0; i<0; ++i,it++) { } assertTrue( *it == "2012" );
    it = LLL.begin(); for (int i=0; i<1; ++i,it++) { } assertTrue( *it == "." );
    it = LLL.begin(); for (int i=0; i<2; ++i,it++) { } assertTrue( *it == "1" );

    it = LLL.begin(); for (int i=0; i<8  ; ++i,it++) { } assertTrue( *it == "adolfo" );
    it = LLL.begin(); for (int i=0; i<19 ; ++i,it++) { } assertTrue( *it == "htm" );

    it = LLL.begin(); for (int i=0; i<9 ; ++i,it++) { } assertTrue( *it == "." );
    it = LLL.begin(); for (int i=0; i<10; ++i,it++) { } assertTrue( *it == "." );
    assertTrue( "htm" == LLL.back() );
    assertTrue( 20 == LLL.size() );
}

{   // Ms pruebas para tokens()
    std::string ln;
    ln = "  gsec =##-tarea|##-tpg (enunciado mejor peor)  ;?-###   ;?###  ;etc.;" ;
    std::list< std::string > LLL;
    std::list< std::string >::const_iterator it;
    tokens( ln, LLL , " =();|" );
    assertTrue( ";" == LLL.back() );
}

{   // Ms pruebas para tokens()
}

{   // Otras pruebas para tokens()
    std::string ln;
    std::list< std::string > LLL;
    std::list< std::string >::const_iterator it;
    const char* delimiters = " /";
    ln = "SDF/Algor   Design";
    tokens( ln, LLL , delimiters );
    it = LLL.begin();
    assertTrue( *it == "SDF");
    ++it; assertTrue( *it == "/");
    ++it; assertTrue( *it == "Algor");
    ++it; assertTrue( *it == "Design");
    ++it; assertTrue( it == LLL.end() );
}

{   // Otras pruebas para tokens()
    std::string ln;
    std::list< std::string > LLL;
    std::list< std::string >::const_iterator it;
    {   //    12345 6 7 8 9 0 1   2  3  4  5
        ln = "()=;| ( ) = ; | (   )  =  ;  |";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==15 );
        assertTrue( LLL.front()=="(" ); assertTrue( LLL.back()=="|" );
        it = LLL.begin(); for (int i=1; i<10; ++i,it++) { } assertTrue( *it == "|" );

        ln = "  "+ln;
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==15 );
        assertTrue( LLL.front()=="(" ); assertTrue( LLL.back()=="|" );
        it = LLL.begin(); for (int i=1; i<10; ++i,it++) { } assertTrue( *it == "|" );

        ln = ln+"  ";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==15 );
        assertTrue( LLL.front()=="(" ); assertTrue( LLL.back()=="|" );
        it = LLL.begin(); for (int i=1; i<10; ++i,it++) { } assertTrue( *it == "|" );
    }
    {   //    123456789 012 3 45 67 89 01 23 45 67 89
        ln = "(.).=.;.|..(..)..=..;..|..(..)..=..;..|";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==29 );
        assertTrue( LLL.front()=="(" ); assertTrue( LLL.back()=="|" );
        it = LLL.begin(); for (int i=1; i<15; ++i,it++) { } assertTrue( *it == "=" );

        ln = ".."+ln;
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==30 );
        assertTrue( LLL.front()==".." ); assertTrue( LLL.back()=="|" );
        it = LLL.begin(); for (int i=1; i<16; ++i,it++) { } assertTrue( *it == "=" );

        ln = ln+".";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==31 );
        assertTrue( LLL.front()==".." ); assertTrue( LLL.back()=="." );
        it = LLL.begin(); for (int i=1; i<16; ++i,it++) { } assertTrue( *it == "=" );
    }
    {   // ln.replace(".",":::");
        ln = "(:::):::=:::;:::|::::::(::::::)::::::=::::::;::::::|::::::(::::::)::::::=::::::;::::::|";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==29 );
        assertTrue( LLL.front()=="(" ); assertTrue( LLL.back()=="|" );
        it = LLL.begin(); for (int i=1; i<15; ++i,it++) { } assertTrue( *it == "=" );

        ln = "---"+ln;
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==30 );
        assertTrue( LLL.front()=="---" ); assertTrue( LLL.back()=="|" );
        it = LLL.begin(); for (int i=1; i<16; ++i,it++) { } assertTrue( *it == "=" );

        ln = ln+"---";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==31 );
        assertTrue( LLL.front()=="---" ); assertTrue( LLL.back()=="---" );
        it = LLL.begin(); for (int i=1; i<16; ++i,it++) { } assertTrue( *it == "=" );
    }
    {
        ln = "";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==0 );
        ln = ";";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==1 );
        ln = " ;  ";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==1 );
        ln = ";  =";
        LLL.clear(); tokens( ln, LLL , " =();|" );
        assertTrue( LLL.size()==2 );
        assertTrue( LLL.front()==";" ); assertTrue( LLL.back()=="=" );
    }
} // Otras pruebas para tokens()
} // test_tokens()


/// Programa de prueba para las rutinas de  \c string_tool.h.
int main( int argc , const char* argv[] ) {
{{{
    // Pone el rtulo de la prueba
    std::cout << "test->string_tool() [uUnit.h]" << std::endl;
}}}
    test_tolower();
    test_removeAccented();
    test_isalpha();
    test_is_blank();
    test_digitCount();
    test_alphaCount();
    test_roundUp();
    test_issuffix();
    test_str2uint();
    test_number_suffix();
    test_tostring();
    test_is_email();
    test_tokens();
    test_psz_string();

    return 0;
}

// EOF: test_tool.cpp
